"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    SchedulerEvent: ()=>SchedulerEvent,
    SchedulerTarget: ()=>SchedulerTarget,
    SchedulerEventGroup: ()=>SchedulerEventGroup,
    SchedulerEventGroupNode: ()=>SchedulerEventGroupNode
});
const _everydayUtils = require("everyday-utils");
const _immutableMapSet = require("immutable-map-set");
const _midiEvents = require("webaudio-tools/midi-events");
const _messageQueue = require("./message-queue");
let SchedulerEvent = class SchedulerEvent {
    id = (0, _everydayUtils.cheapRandomId)();
    midiEvent = {
        data: new Uint8Array(3),
        receivedTime: 0
    };
    constructor(data = {}){
        this.id = data.id ?? this.id;
        if (data.midiEvent) {
            this.midiEvent.data = data.midiEvent.data;
            this.midiEvent.receivedTime = data.midiEvent.receivedTime;
        }
    }
    toJSON() {
        return {
            id: this.id,
            midiEvent: {
                receivedTime: this.midiEvent.receivedTime ?? 0,
                data: this.midiEvent.data
            }
        };
    }
};
let SchedulerTarget = class SchedulerTarget {
    id = (0, _everydayUtils.cheapRandomId)();
    midiQueue = new _messageQueue.MessageQueue().clear();
    constructor(data = {}){
        Object.assign(this, data);
    }
};
let SchedulerEventGroup = class SchedulerEventGroup {
    id = (0, _everydayUtils.cheapRandomId)();
    targets = new _immutableMapSet.ImmSet();
    events = new _immutableMapSet.ImmSet();
    loopPoints = new Float64Array(new SharedArrayBuffer(3 // [loopActive, loopStart, loopEnd]
     * Float64Array.BYTES_PER_ELEMENT));
    constructor(eventGroup = {}){
        Object.assign(this, eventGroup);
    }
    toJSON() {
        return {
            id: this.id,
            targets: this.targets,
            events: this.events,
            loopPoints: this.loopPoints
        };
    }
    get loop() {
        return !!this.loopPoints[0];
    }
    set loop(value) {
        this.loopPoints[0] = +value;
    }
    get loopStart() {
        return this.loopPoints[1];
    }
    set loopStart(seconds) {
        this.loopPoints[1] = seconds;
    }
    get loopEnd() {
        return this.loopPoints[2];
    }
    set loopEnd(seconds) {
        this.loopPoints[2] = seconds;
    }
    replaceAllWithNotes(notes) {
        const midiEvents = [];
        for (const [time, note, velocity, length] of notes){
            midiEvents.push(...(0, _midiEvents.createMidiNoteEvents)(time, note, velocity, length));
        }
        this.events = this.events.clear();
        for (const midiEvent of midiEvents){
            const event = new SchedulerEvent({
                midiEvent
            });
            this.events = this.events.add(event);
        }
        return midiEvents;
    }
};
let SchedulerEventGroupNode = class SchedulerEventGroupNode {
    eventGroup;
    constructor(schedulerNode){
        this.schedulerNode = schedulerNode;
        this.eventGroup = new SchedulerEventGroup();
        this.schedulerNode.eventGroups.add(this.eventGroup);
    }
    destroy() {
        this.schedulerNode.eventGroups.delete(this.eventGroup);
    }
    connect(targetNode) {
        this.schedulerNode.connect(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.add(targetNode.schedulerTarget);
        return targetNode;
    }
    disconnect(targetNode) {
        this.schedulerNode.disconnect(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.delete(targetNode.schedulerTarget);
    }
    schedulerNode;
};

//# sourceMappingURL=scheduler-event.js.map