"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    MIDIMessageEvent: ()=>_midi.MIDIMessageEvent,
    SchedulerTargetProcessor: ()=>SchedulerTargetProcessor
});
const _aliceBob = require("alice-bob");
const _messageQueue = require("./message-queue");
const _midi = require("./midi");
const sortByFrame = (a, b)=>a.offsetFrame - b.offsetFrame;
let SchedulerTargetProcessor = class SchedulerTargetProcessor extends AudioWorkletProcessor {
    midiQueue = new _messageQueue.MessageQueue();
    constructor(){
        super();
        const [worklet] = new _aliceBob.Bob((data)=>this.port.postMessage(data), this).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>worklet.receive(data);
    }
    async init(buffer) {
        this.midiQueue.buffer = buffer;
    }
    process(inputs, outputs, parameters) {
        // TODO: optimizations possible:
        //  reuse midiEvents
        //  events pool
        //  sort by hash?
        const midiEvents = [];
        let message;
        let prevFrame = 0;
        while(message = this.midiQueue.slice(4)){
            const event = new _midi.MIDIMessageEvent('midimessage', {
                data: new Uint8Array(message.slice(1))
            });
            // receivedTime === 0 signifies "process as soon as possible"
            // so we give it our currentTime
            event.receivedTime = message[0] || currentTime * 1000;
            event.receivedFrame = Math.floor(event.receivedTime * 0.001 * sampleRate);
            event.offsetFrame = event.receivedFrame - currentFrame;
            event.deltaFrame = Math.max(0, event.offsetFrame - prevFrame);
            prevFrame = event.offsetFrame;
            midiEvents.push(event);
        }
        if (midiEvents.length) {
            midiEvents.sort(sortByFrame);
            this.processMidiEvents(midiEvents);
        }
        return this.processWithMidi(inputs, outputs, parameters, midiEvents);
    }
};

//# sourceMappingURL=scheduler-target-processor.js.map