import { cheapRandomId } from 'everyday-utils';
import { ImmSet } from 'immutable-map-set';
import { createMidiNoteEvents } from 'webaudio-tools/midi-events';
import { MessageQueue } from './message-queue';
export class SchedulerEvent {
    id = cheapRandomId();
    midiEvent = {
        data: new Uint8Array(3),
        receivedTime: 0
    };
    constructor(data = {}){
        this.id = data.id ?? this.id;
        if (data.midiEvent) {
            this.midiEvent.data = data.midiEvent.data;
            this.midiEvent.receivedTime = data.midiEvent.receivedTime;
        }
    }
    toJSON() {
        return {
            id: this.id,
            midiEvent: {
                receivedTime: this.midiEvent.receivedTime ?? 0,
                data: this.midiEvent.data
            }
        };
    }
}
export class SchedulerTarget {
    id = cheapRandomId();
    midiQueue = new MessageQueue().clear();
    constructor(data = {}){
        Object.assign(this, data);
    }
}
export class SchedulerEventGroup {
    id = cheapRandomId();
    targets = new ImmSet();
    events = new ImmSet();
    loopPoints = new Float64Array(new SharedArrayBuffer(3 // [loopActive, loopStart, loopEnd]
     * Float64Array.BYTES_PER_ELEMENT));
    constructor(eventGroup = {}){
        Object.assign(this, eventGroup);
    }
    toJSON() {
        return {
            id: this.id,
            targets: this.targets,
            events: this.events,
            loopPoints: this.loopPoints
        };
    }
    get loop() {
        return !!this.loopPoints[0];
    }
    set loop(value) {
        this.loopPoints[0] = +value;
    }
    get loopStart() {
        return this.loopPoints[1];
    }
    set loopStart(seconds) {
        this.loopPoints[1] = seconds;
    }
    get loopEnd() {
        return this.loopPoints[2];
    }
    set loopEnd(seconds) {
        this.loopPoints[2] = seconds;
    }
    replaceAllWithNotes(notes) {
        const midiEvents = [];
        for (const [time, note, velocity, length] of notes){
            midiEvents.push(...createMidiNoteEvents(time, note, velocity, length));
        }
        this.events = this.events.clear();
        for (const midiEvent of midiEvents){
            const event = new SchedulerEvent({
                midiEvent
            });
            this.events = this.events.add(event);
        }
        return midiEvents;
    }
}
export class SchedulerEventGroupNode {
    eventGroup;
    constructor(schedulerNode){
        this.schedulerNode = schedulerNode;
        this.eventGroup = new SchedulerEventGroup();
        this.schedulerNode.eventGroups.add(this.eventGroup);
    }
    destroy() {
        this.schedulerNode.eventGroups.delete(this.eventGroup);
    }
    connect(targetNode) {
        this.schedulerNode.connect(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.add(targetNode.schedulerTarget);
        return targetNode;
    }
    disconnect(targetNode) {
        this.schedulerNode.disconnect(targetNode);
        this.eventGroup.targets = this.eventGroup.targets.delete(targetNode.schedulerTarget);
    }
    schedulerNode;
}

//# sourceMappingURL=scheduler-event.js.map