import { Alice } from 'alice-bob';
import { cheapRandomId } from 'everyday-utils';
import { SchedulerTarget } from './scheduler-event';
export class SchedulerTargetNode extends AudioWorkletNode {
    id = cheapRandomId();
    schedulerTarget = new SchedulerTarget();
    worklet;
    constructor(context, name, options = {}){
        // add a phony tail input, used for connecting the
        // scheduler to execute right before our worklet
        options.numberOfInputs = (options.numberOfInputs ?? 0) + 1;
        super(context, name, options);
        const [node, worklet] = new Alice((data)=>void this.port.postMessage(data)).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>node.receive(data);
        // TODO: handle onprocessorerror
        this.worklet = worklet;
    }
    async init() {
        await this.worklet.init(this.schedulerTarget.midiQueue.buffer);
    }
    processMidiEvent(midiEvent) {
        if (midiEvent.receivedTime == null) {
            throw new Error('Midi event missing receivedTime');
        }
        this.schedulerTarget.midiQueue.push(midiEvent.receivedTime, ...midiEvent.data);
    }
}

//# sourceMappingURL=scheduler-target-node.js.map