import { Bob } from 'alice-bob';
import { MessageQueue } from './message-queue';
import { MIDIMessageEvent } from './midi';
export { MIDIMessageEvent };
const sortByFrame = (a, b)=>a.offsetFrame - b.offsetFrame;
export class SchedulerTargetProcessor extends AudioWorkletProcessor {
    midiQueue = new MessageQueue();
    constructor(){
        super();
        const [worklet] = new Bob((data)=>this.port.postMessage(data), this).agents({
            debug: false
        });
        this.port.onmessage = ({ data  })=>worklet.receive(data);
    }
    async init(buffer) {
        this.midiQueue.buffer = buffer;
    }
    process(inputs, outputs, parameters) {
        // TODO: optimizations possible:
        //  reuse midiEvents
        //  events pool
        //  sort by hash?
        const midiEvents = [];
        let message;
        let prevFrame = 0;
        while(message = this.midiQueue.slice(4)){
            const event = new MIDIMessageEvent('midimessage', {
                data: new Uint8Array(message.slice(1))
            });
            // receivedTime === 0 signifies "process as soon as possible"
            // so we give it our currentTime
            event.receivedTime = message[0] || currentTime * 1000;
            event.receivedFrame = Math.floor(event.receivedTime * 0.001 * sampleRate);
            event.offsetFrame = event.receivedFrame - currentFrame;
            event.deltaFrame = Math.max(0, event.offsetFrame - prevFrame);
            prevFrame = event.offsetFrame;
            midiEvents.push(event);
        }
        if (midiEvents.length) {
            midiEvents.sort(sortByFrame);
            this.processMidiEvents(midiEvents);
        }
        return this.processWithMidi(inputs, outputs, parameters, midiEvents);
    }
}

//# sourceMappingURL=scheduler-target-processor.js.map